<?php

ini_set('max_execution_time', 300); //300 seconds = 5 minutes

require_once('includes/PasswordHash.php');
require_once('includes/_dbconfig.php');
require __DIR__ . '/vendor/autoload.php';


date_default_timezone_set('Africa/Nairobi');

use Firebase\JWT\JWT;

$app = new \Slim\App([
'settings' => [
    'determineRouteBeforeAppMiddleware' => true,
    'displayErrorDetails' => true,
    'addContentLengthHeader' => false
]
]);

$app->get('/safaricom/staff/token/{id}', 'getStaffToken');
$app->post('/safaricom/staff/login/', 'staffLogin');
$app->get('/safaricom/password/{password}', 'getPassword');

$app->post('/safaricom/backup/staff/', 'backupStaff');
$app->post('/safaricom/backup/scores/', 'backupScores');

$app->run();



/**
 * Function Name: backupScores
 *
 * Adds a new score
 *
 * @param (array) ([staff details]) the staff details
 * @return (number) ($rows) the number of rows added
 *
 */
function backupScores($request, $response, $args) {
	
	$score = json_decode($request->getBody());
	
	$sql = "INSERT INTO scores (staff_name, staff_phone, staff_email, points, last_played) VALUES (:staff_name, :staff_phone, :staff_email, :points, :last_played)";
	    
	    try {
	    	
	    	$staff_name = $score->staff_name;
	    	$staff_phone = $score->staff_phone;
	    	$staff_email = $score->staff_email;	        
	        $points = strval($score->points);	        
	        $time = strtotime($score->last_played);
	        $last_played = date ("Y-m-d H:i:s", $time);

	    	$db = getConnection();
			$stmt = $db->prepare($sql);
			
			$stmt->bindParam("staff_name", $staff_name);
			$stmt->bindParam("staff_phone", $staff_phone);
			$stmt->bindParam("staff_email", $staff_email);
			$stmt->bindParam("points", $points);
			$stmt->bindParam("last_played", $last_played);
			
			$rows = $stmt->execute();					

			$db = null;

			if($rows > 0){
				
				$data = array('success' => true, 'message' => $staff_name .' score has been added to DB');
				return $response->withJson($data,200);			    
			    
			}else{
		    	
		    	$data = array('success' => false, 'message' => 'Sorry '.$guest_name .', unable to process your request');
				return $response->withJson($data,200);
		    }
	    
		    
	    
	    } catch(PDOException $e) {		
		     
		    $data = array('success' => false, 'message' => $e->getMessage());			
			return $response->withJson($data,200);
	    }		   
	
}




/**
 * Function Name: backupStaff
 *
 * Adds a new staff
 *
 * @param (array) ([staff details]) the staff details
 * @return (number) ($rows) the number of rows added
 *
 */
function backupStaff($request, $response, $args) {
	
	$staff = json_decode($request->getBody());
	
	$sql = "INSERT INTO staff (staff_name, staff_phone, staff_email) VALUES (:staff_name, :staff_phone, :staff_email)";
	    
	    try {

	    	
	    	$staff_name = $staff->staff_name;
	    	$staff_phone = $staff->staff_phone;
	    	$staff_email = $staff->staff_email;

	    	$staff_name = cleanField($staff_name);
	    	$staff_phone = cleanField($staff_phone);	    			    	
	    	$db = getConnection();
			$stmt = $db->prepare($sql);
			
			$stmt->bindParam("staff_name", $staff_name);
			$stmt->bindParam("staff_phone", $staff_phone);
			$stmt->bindParam("staff_email", $staff_email);
			
			$rows = $stmt->execute();					

			$db = null;

			if($rows > 0){
				
				$data = array('success' => true, 'message' => $staff_name .' has been added to DB');
				return $response->withJson($data,200);			    
			    
			}else{
		    	
		    	$data = array('success' => false, 'message' => 'Sorry '.$guest_name .', unable to process your request');
				return $response->withJson($data,200);
		    }
	    
		    
	    
	    } catch(PDOException $e) {		
		     
		    $data = array('success' => false, 'message' => $e->getMessage());			
			return $response->withJson($data,200);
	    }		   
	
}





/**
 * Function Name: getPassword
 *
 * Generate a hash password
 *
 * @param (array) ([password]) the text password 
 * @return (text) ($pass_word) the hash password
 *
 */
function getPassword($request, $response, $args) {
	
	$password = $args['password'];
	$hasher = new PasswordHash(8, FALSE);
	$pass_word = $hasher->HashPassword($password);
	
	$data = array('success' => true, 'password' => $pass_word);    
	return $response->withJson($data,200);		   
	
}



/**
 * Function Name: getStaff
 *
 * Retrieves all staff users details as json from firebase
 * 
 * @return (array) ($users) the actual users details
 */
function getStaff($url) {	
	
	$default_firebase_url = 'https://sdgs-dev.firebaseio.com'; //main firebase url		
	
	$firebase = new \Firebase\FirebaseLib($default_firebase_url);
	
	$users = $firebase->get('/'.$url);	//all user details from firebase

	return $users;
}






/**
 * Function Name: staffLogin
 *
 * Login to the app so it verifys if the entered details are correct
 * 
 * @return (object) ($token) the actual user details and the token generated
 */
function staffLogin($request, $response, $args) {

	$staff = json_decode($request->getBody());

	//$secret = 'AIzaSyA8o0CYglGaWowSCN_Tw8VVrFtoStZkD0s'; //secret key from firebase
	
	$all_users = getStaff('staff'); //returns json

	$email = $staff->staff_email;
	$password = $staff->staff_password;
	
	$array = json_decode($all_users, true);

	$password_ok = false;
	$user_found = false;
	$logged_in_user = [];
	
	foreach ($array as $user) {

	    if($email == $user['staff_email']){

	    	//check if password match
	    	$user_found = true;
	    	$hash = $user['staff_password'];
	    	$logged_in_user = $user;
	    	$id = key($array); //Get the ID of the current user
	    	
	    	break;
	    }
	}

	if($user_found){

		$hasher = new PasswordHash(8, FALSE);
    	$password_ok = $hasher->CheckPassword($password, $hash);    	
    	
    	if($password_ok){
    		$password_ok = true;
    	}
		
	}
	
	

	//User found
	if($user_found){
		
		//password ok
		if($password_ok){

			$details = (object) $logged_in_user;			

			if($details != null){

				unset($details->staff_password);
			    $token = getToken($id, $details);//Get token for the user

			    $data = array('success' => true, 'token' => $token, 'staff' => $details);

			    return $response->withJson($data,200);

			}else{

				$data = array('success' => false, 'error' => $e->getMessage());
			    return $response->withJson($data,200);

			}
	        

		}else{ //password NOT ok
			$data = array('success' => false, 'error' => 'Invalid Password');
	        return $response->withJson($data,200);
		}


	}else{ //User not found
		$data = array('success' => false, 'error' => 'Invalid Username');
	    return $response->withJson($data,200);
	}


}




/**
 * Function Name: getStaffToken
 *
 * Retrieves all user details from firebase and generates a secret token
 * 
 * @return (object) ($token) the actual user details token
 */
function getStaffToken($request, $response, $args) {
	
	$users_url = '/staff'; //the firebase users url
	$default_firebase_url = 'https://sdgs-dev.firebaseio.com'; //main firebase url
	
	$firebase = new \Firebase\FirebaseLib($default_firebase_url);

	$id = $args['id'];
	$user = $firebase->get($users_url.'/'.$id);	//user details from firebase
	$details = json_decode($user);
	
	try {
	    

	    function create_custom_token($uid, $staff) {

	    $service_account_email = "william@sdgs-dev.iam.gserviceaccount.com";
	    $private_key = "-----BEGIN PRIVATE KEY-----\nMIIEvgIBADANBgkqhkiG9w0BAQEFAASCBKgwggSkAgEAAoIBAQDPFy557SixQGGi\niQa9WTnKYJ/wYGCzaRX8cHofNkIFkt55V4sfsVE2O8asyP6/cPDoykEq6/YjEwXy\n+cVvdQqXal0x8jacXqqwgmhG+rJEcqkkMz5Z137TcQ1gdUNHa4zOTjspNfeFCTfl\nZyf2bKkW0FTPsErexvP8Q6V/kb8db6wxc6PxTavf8dndruQQ7vvN261arJED3py9\nlOqxL0irdc1+Fv1MaAbF5bv94H8cO9P4DPuRXDQ0Mk78nTRjrbELMXvnWb1AYUXu\nes7YclXnjBNtZgD/h+otN+kJE9jj+sewZusEWXUnm/jIq3SJdll1ywf9p6d9ZexW\nG4yF/V21AgMBAAECggEBAI16c9j+ac92gX1OCdmwLGgTTREmdsn7H/cfTH8oTUCL\njwvs//H97NKVUezXQBq/6QXQr2bIX5akt2rAst6rYjoBg3vM0vxPPGNsJBthyAo1\nH1LI427zZwTC9nWQhw8y7bZAAfDsmpY7rwE9UTLsFzSrzVU6RlweJa+Q827Mvnk3\niAuWHAGfgnSeaciD0PCTwhv5A5CduYEYpbiOQzQf54oPum6AXXMpzDs/u6W//J9R\nvbxvvBe2+qboI+He7ZT6UdRgEnr9E2fqTY2Jvd6CmJpimLGzOnIdHiYMhzLxFRPq\nAibthBNtg8WRNrmZeXmK6lspSQNl1QTkptWoADqPG/0CgYEA6vV/o8LiITGKnXvi\n+P+3rSLy6oHWlpOskH97YuXgdTGuAjSJOx/0afTW9Y0kMNNjveON8TDA7VCNbV2W\nPszEgrCO86Rig/M6Je6jmx+r5YdI2zhaxbkofDjlCHdp2mljDg9qDhHV1w1g9gAo\nkyVpB2zoDpDUStts3a0jRHZB5WMCgYEA4aLKlunoQxyXJe3MKVewVhizTY+xZrjA\nrXy29r6aPTmwWTwCdSoJjValoePOmD3dLLobj3dokHLRN8O1H+PKw4r3qhZbx4HT\nhIScQuaiaTBepaYLOZj/GfRJGqz1eUXc/rvVmuhoU2H3GvzLBUfrJuB4U4EJ6Bhw\nwPooKB53CAcCgYEAyUtofhFRkGSUQBXKP889OmzNtpFkGf8MgYNmXzVYVq8oSCVg\nSLI4wKoNm7O/BMGmGyIgEjnVn7s6Xy8lsEnYNasz3ZLBABDaiXLo4daBe5Sqm32t\nbuYRxFMuDVHY81EElyAnmUQ4cxS4NCmwmVgcokrwiMIVzsgsfcJ+i4HRNMcCgYAg\neuii1MUEdCcl5JdRFDhkDGYzMg4ddVb3Ik7MMh/N1HIYRlUsPILnf6OULFDfBXjD\nKX8yJcQ9lumbF2cJUicsAuMn7tQDYiLqnpRH5R9X+QiG2+IdxnJicBHd7yve6zmm\nya8z/2SVLDMABYfMGNA25nUrwBWG/Y2tmdUSAWCtHQKBgHRzGRDRzwruFTafgY6P\nTamuWtT2w7l0WA7NkjQtewBNIiNztzoViCT04ujzhUxK7/ZucTmXtiwSZnea5gSi\nRgyKr9DS42QHhbG2Psi/IuBAyEzeS32j3z0WD2ezLZVs2XhUvNIVZMunrCsFakPe\njh6Cer7Ua8KNpYaL1ohlRwaF\n-----END PRIVATE KEY-----\n";    
		  	
		  	  //global $service_account_email, $private_key;

			  $now_seconds = time();

			  $payload = array(
			    "iss" => $service_account_email,
			    "sub" => $service_account_email,
			    "aud" => "https://identitytoolkit.googleapis.com/google.identity.identitytoolkit.v1.IdentityToolkit",
			    "iat" => $now_seconds,
			    "exp" => $now_seconds+(60*60),  // Maximum expiration time is one hour
			    "uid" => $uid,
			    "claims" => [
			        'uid' => $uid,
			        'staff_name' => $staff->staff_name,
			        'staff_email' => $staff->staff_email,
			        'staff_phone' => $staff->staff_phone
			    ],
			  );			  

			  return JWT::encode($payload, $private_key, "RS256");
		}

		$token = create_custom_token($id, $details);		
		unset($details->staff_password);
	    $data = array('success' => true, 'token' => $token, 'staff' => $details);
	    
	    return $response->withJson($data,200);

	} catch (Exception $e) {
	   
	    $data = array('success' => false, 'error' => $e->getMessage());
	    return $response->withJson($data,200);
	}

}




/**
 * Function Name: getToken
 *
 * Retrieves generates a secret token
 * 
 * @return (object) ($token) the actual user details token
 */
function getToken($id, $user) {	
		
	try {	    

	    function create_custom_token($uid, $staff) {

	    	$service_account_email = "william@sdgs-dev.iam.gserviceaccount.com";
	    	$private_key = "-----BEGIN PRIVATE KEY-----\nMIIEvgIBADANBgkqhkiG9w0BAQEFAASCBKgwggSkAgEAAoIBAQDPFy557SixQGGi\niQa9WTnKYJ/wYGCzaRX8cHofNkIFkt55V4sfsVE2O8asyP6/cPDoykEq6/YjEwXy\n+cVvdQqXal0x8jacXqqwgmhG+rJEcqkkMz5Z137TcQ1gdUNHa4zOTjspNfeFCTfl\nZyf2bKkW0FTPsErexvP8Q6V/kb8db6wxc6PxTavf8dndruQQ7vvN261arJED3py9\nlOqxL0irdc1+Fv1MaAbF5bv94H8cO9P4DPuRXDQ0Mk78nTRjrbELMXvnWb1AYUXu\nes7YclXnjBNtZgD/h+otN+kJE9jj+sewZusEWXUnm/jIq3SJdll1ywf9p6d9ZexW\nG4yF/V21AgMBAAECggEBAI16c9j+ac92gX1OCdmwLGgTTREmdsn7H/cfTH8oTUCL\njwvs//H97NKVUezXQBq/6QXQr2bIX5akt2rAst6rYjoBg3vM0vxPPGNsJBthyAo1\nH1LI427zZwTC9nWQhw8y7bZAAfDsmpY7rwE9UTLsFzSrzVU6RlweJa+Q827Mvnk3\niAuWHAGfgnSeaciD0PCTwhv5A5CduYEYpbiOQzQf54oPum6AXXMpzDs/u6W//J9R\nvbxvvBe2+qboI+He7ZT6UdRgEnr9E2fqTY2Jvd6CmJpimLGzOnIdHiYMhzLxFRPq\nAibthBNtg8WRNrmZeXmK6lspSQNl1QTkptWoADqPG/0CgYEA6vV/o8LiITGKnXvi\n+P+3rSLy6oHWlpOskH97YuXgdTGuAjSJOx/0afTW9Y0kMNNjveON8TDA7VCNbV2W\nPszEgrCO86Rig/M6Je6jmx+r5YdI2zhaxbkofDjlCHdp2mljDg9qDhHV1w1g9gAo\nkyVpB2zoDpDUStts3a0jRHZB5WMCgYEA4aLKlunoQxyXJe3MKVewVhizTY+xZrjA\nrXy29r6aPTmwWTwCdSoJjValoePOmD3dLLobj3dokHLRN8O1H+PKw4r3qhZbx4HT\nhIScQuaiaTBepaYLOZj/GfRJGqz1eUXc/rvVmuhoU2H3GvzLBUfrJuB4U4EJ6Bhw\nwPooKB53CAcCgYEAyUtofhFRkGSUQBXKP889OmzNtpFkGf8MgYNmXzVYVq8oSCVg\nSLI4wKoNm7O/BMGmGyIgEjnVn7s6Xy8lsEnYNasz3ZLBABDaiXLo4daBe5Sqm32t\nbuYRxFMuDVHY81EElyAnmUQ4cxS4NCmwmVgcokrwiMIVzsgsfcJ+i4HRNMcCgYAg\neuii1MUEdCcl5JdRFDhkDGYzMg4ddVb3Ik7MMh/N1HIYRlUsPILnf6OULFDfBXjD\nKX8yJcQ9lumbF2cJUicsAuMn7tQDYiLqnpRH5R9X+QiG2+IdxnJicBHd7yve6zmm\nya8z/2SVLDMABYfMGNA25nUrwBWG/Y2tmdUSAWCtHQKBgHRzGRDRzwruFTafgY6P\nTamuWtT2w7l0WA7NkjQtewBNIiNztzoViCT04ujzhUxK7/ZucTmXtiwSZnea5gSi\nRgyKr9DS42QHhbG2Psi/IuBAyEzeS32j3z0WD2ezLZVs2XhUvNIVZMunrCsFakPe\njh6Cer7Ua8KNpYaL1ohlRwaF\n-----END PRIVATE KEY-----\n";    
		  	
		  	  //global $service_account_email, $private_key;

			  $now_seconds = time();

			  $payload = array(
			    "iss" => $service_account_email,
			    "sub" => $service_account_email,
			    "aud" => "https://identitytoolkit.googleapis.com/google.identity.identitytoolkit.v1.IdentityToolkit",
			    "iat" => $now_seconds,
			    "exp" => $now_seconds+(60*60),  // Maximum expiration time is one hour
			    "uid" => $uid,
			    "claims" => [
			        'uid' => $uid,
			        'staff_name' => $staff->staff_name,
			        'staff_email' => $staff->staff_email,
			        'staff_phone' => $staff->staff_phone
			    ],
			  );			  

			  return JWT::encode($payload, $private_key, "RS256");

		}//End of create_custom_token

		$token = create_custom_token($id, $user);		
		
	    return $token;

	} catch (Exception $e) {
	   
	    $data = array('success' => false, 'error' => $e->getMessage());
	    return $response->withJson($data,200);
	}

}





function checkEmail($str){
	return preg_match("/^[\.A-z0-9_\-\+]+[@][A-z0-9_\-]+([.][A-z0-9_\-]+)+[A-z]{1,4}$/", $str);
}



function cleanField($text){
	
	$name = preg_replace("/[^a-zA-Z0-9\/_|+,<>\[\]() .-]/", '', $text);//remove special characters
	$name = preg_replace("/[\"\']/", '', $name); //remove any quotes

	return $name;
}



?>