<?php

ini_set('max_execution_time', 300); //300 seconds = 5 minutes
date_default_timezone_set('Africa/Nairobi');

require_once('includes/_dbconfig.php');
require __DIR__ . '/vendor/autoload.php';

$app = new \Slim\App();


$app->post('/dow/guest/', 'addGuest'); //Add a new service
$app->get('/dow/guest/', 'getGuests'); //Get all the guests from the DB
$app->put('/dow/guest/{id}', 'updateGuest'); //Get all the guests from the DB



$app->run();


/**
 * Function Name: getGuests
 *
 * Retrieves all guests 
 * 
 * @return (object) ($token) the actual guests details 
 */
function getGuests($request, $response, $args) {	

	$sql = "SELECT id,guest_name,guest_phone, guest_company FROM guests ORDER BY guest_name";

	try {
		$db = getConnection();
		$stmt = $db->prepare($sql);	
		$stmt->execute();				  
		$guests = $stmt->fetchAll(PDO::FETCH_OBJ); 	 
		$db = null;
		
		/*$data = array('success' => true, 'guests' => $guests);
		return $response->withJson($data,200);*/

		ob_clean();

		$data = array('success' => true, 'guests' => $guests);
		
		$body = $response->getBody();
		$body->write(json_encode($data, JSON_UNESCAPED_SLASHES));
		$k = $body->getSize();
		echo '   ';
		
	} catch(PDOException $e) {		
		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());

		return $response->withJson($data,200);

	}
}	







/**
 * Function Name: addGuest
 *
 * Adds a new guest
 *
 * @param (array) ([contact details]) the guest details
 * @return (number) ($rows) the number of rows added
 *
 */
function addGuest($request, $response, $args) {
	
	$guest = json_decode($request->getBody());
	
	$sql = "INSERT INTO guests (guest_name, guest_phone, guest_company, checked_in) VALUES (:guest_name, :guest_phone, :guest_company, :checked_in)";
	    
	    try {

	    	$checked_in = '1';
	    	$guest_name = $guest->guest_name;
	    	$guest_phone = $guest->guest_phone;
	    	$guest_company = $guest->guest_company;

	    	$guest_name = cleanField($guest_name);
	    	$guest_phone = cleanField($guest_phone);
	    	$guest_company = cleanField($guest_company);
	    			    	
	    	$db = getConnection();
			$stmt = $db->prepare($sql);
			
			$stmt->bindParam("guest_name", $guest_name);
			$stmt->bindParam("guest_phone", $guest_phone);
			$stmt->bindParam("guest_company", $guest_company);
			$stmt->bindParam("checked_in", $checked_in);
			
			$rows = $stmt->execute();					

			$db = null;

			if($rows > 0){
				
				$data = array('success' => true, 'message' => 'Thank you '.$guest_name .', for registering.');
				return $response->withJson($data,200);			    
			    
			}else{
		    	
		    	$data = array('success' => false, 'message' => 'Sorry '.$guest_name .', unable to process your request');
				return $response->withJson($data,200);
		    }
	    
		    
	    
	    } catch(PDOException $e) {		
		     
		    $data = array('success' => false, 'message' => $e->getMessage());			
			return $response->withJson($data,200);
	    }		   
	
}





/**
 * Function Name: updateGuest
 *
 * Update a specific guest by ID
 *
 * @param (number) (id) this is the id of the guest
 * @return (number) (rows) the number of db rows deleted
 */
function updateGuest($request, $response, $args) {	

	$id = $args['id'];

	$guest = json_decode($request->getBody());

	$checked_in = cleanField($guest->checked_in);
	
	
	$sql = "UPDATE guests SET checked_in =:checked_in WHERE id =:id";

	try {
		$db = getConnection();
		$db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		$stmt = $db->prepare($sql);
		$stmt->bindParam("id", $id);
		$stmt->bindParam("checked_in", $checked_in);
		
		$stmt->execute();
		$rows = $stmt->rowCount();	
		$db = null;
		
		if($rows > 0){
		    $data = array('success' => true, 'message' => 'Thank you for checking in');
			return $response->withJson($data,200);
		}else{
			$data = array('success' => false, 'message' => 'Sorry guest record failed to edit');
			return $response->withJson($data,200);
		}
		
	} catch(PDOException $e) {		
		$data = array('success' => false, 
					  'message' => $e->getMessage(), 
					  'code' => $e->getCode(),
					  'one_line' => $e->getLine());
		return $response->withJson($data,200);
	}
	
}





function checkEmail($str){
	return preg_match("/^[\.A-z0-9_\-\+]+[@][A-z0-9_\-]+([.][A-z0-9_\-]+)+[A-z]{1,4}$/", $str);
}



function cleanField($text){
	
	$name = preg_replace("/[^a-zA-Z0-9\/_|+,<>\[\]() .-]/", '', $text);//remove special characters
	$name = preg_replace("/[\"\']/", '', $name); //remove any quotes

	return $name;
}



?>