define([
    'angular',
    'jquery',
    'uiRouter',
    'fclick',
    'ocLazyLoad', 
    'fireBase', 
    'angularFire', 
    './controllers',
    './services',
    './directives',
    'angularidle',
    'ngscroll',
    'angularAnimate' 
], function (ng, jQuery, uiRouter, FastClick, ocLazyLoad, Firebase, angularFire, controllers, services, directives, angularidle, ngscroll) {
    'use strict';    

    //Initailize the app and pull together all the dependancies here
    var app = ng.module('app', [        
        'app.main.services',
        'app.main.controllers',
        'app.main.directives',
        'ui.router',
        'oc.lazyLoad',
        'firebase',
        'ngAnimate',
        'ngIdle',
        'duScroll'
    ]);   
    
    app.factory('$exceptionHandler', function(){
        return function(exception, cause){
            console.log(exception, cause);
        };
    });
    
    //Attach fast click plugin on to the app
    app.run(['$rootScope', '$state', '$stateParams', 'Idle', 'properties', 'settings', '$interval', 'safaricom', '$timeout', function($rootScope, $state, $stateParams, Idle, properties, settings, $interval, safaricom, $timeout) {      
        
        Idle.watch();        

        $rootScope.$state = $state;
        $rootScope.$stateParams = $stateParams; 

        $rootScope.$on("$stateChangeError", function(event, toState, toParams, fromState, fromParams, error) {
          // We can catch the error thrown when the $requireAuth promise is rejected
          // and redirect the user back to the login page
          if (error === "AUTH_REQUIRED") {
                
                $state.go("home.login");
          }

        });  

        $rootScope.scores = safaricom.allScores();
        $rootScope.total_scores = 0;

        $rootScope.scores.$loaded().then(function(data){                    
            $rootScope.scores = data;
            $rootScope.total_scores = $rootScope.scores.length;
        });
         

        $rootScope.show_board = function(){ 
            $state.go('home.board');
        };  
         

        $rootScope.goals = properties.goals;

        $rootScope.goal = {};

        $rootScope.showGoal = function(goal){            

            $rootScope.goal = goal;

            $timeout(function(){
                
                var sdgcontent = $('.sdg-content');
                var overlay = $('.overlay');

                sdgcontent.show().css('display', 'block');
                overlay.show().css('display', 'block');                

                $('.sdg-content .closer').on('click', function(){
                    sdgcontent.hide().css('display', 'none');
                    overlay.hide().css('display', 'none');
                });


            });
        };


        $rootScope.landing = function(){
            $state.go('landing');         
        };
        
        FastClick.attach(document.body);

        $rootScope.$on("$stateChangeStart", function(event, toState, toParams, fromState, fromParams){
            
            properties.setCurrentURL(toState.name);           

        });


        $rootScope.$on("$stateChangeSuccess", function(event, toState, toParams, fromState, fromParams){
        
            $rootScope.currenturl = properties.getCurrentURL();           

        });    



    }]);

    return app;
});