define(['./module', 'jquery', 'datejs'], function (controllers, jQuery, Date) {
    'use strict';
    
    controllers.controller("SignupController", ["$scope", "$state", "$timeout", "Idle", "$document", "settings", "safaricom", "properties", function($scope, $state, $timeout, Idle, $document, settings, safaricom, properties) {
	    
      	Idle.watch(); 

      	$scope.authObj = safaricom.auth();	      

      	$scope.$on('IdleTimeout', function() {	        
	        settings.logout();
	    });

	    $scope.$on('IdleStart', function() {
	        settings.notify('Idle timeout, you will be redirected to the home page.');
	    });

	    $scope.toTheTop = function() {

		    $document.scrollTopAnimated(0, 1000).then(function() {
		        //console.log('You just scrolled to the top!');
		    });
	    };

	    $timeout(function () {
	     	$scope.toTheTop();
	    },1000);


	    $scope.allscores = safaricom.allScores();
        
        $scope.allscores.$loaded().then(function(data){                    
            $scope.allscores = data;
        });
	     

	    jQuery(document).on( 'scroll', function(){
 
			if (jQuery(window).scrollTop() > 100) {
				jQuery('.scroll-top-wrapper').addClass('show');
			} else {
				jQuery('.scroll-top-wrapper').removeClass('show');
			}
			
		});	

		$scope.newstaff = {};
		$scope.loginstaff = {};


		$scope.login = function(){
			$state.go('home.login');
		};	

		$scope.register = function(){
			$state.go('home.register');
		};

		$scope.logsign = function(){
			$state.go('home.logsign');
		}


	    $scope.startGame = function(){
		    $state.go('home.play');
	    }

	    $scope.logout = function(){      		
            $scope.authObj.$signOut();
      	};

      	$scope.home = function(){
      		$scope.logout();
			$scope.logsign();
		}


	    
	    $scope.goPlay = function(staffId){
		      	
		      	//get previous scrores of the staff using the ID      	
		      	
		      	$scope.staff_scores = safaricom.allStaffScores(staffId); //$scope.filterStaffScores($scope.scores, staffId);
		      	$scope.staff_scores_length = 0;
		      	
		      	$scope.staff_scores.$loaded().then(function(data){
		    		
		    		$scope.staff_scores = data;	
		    		$scope.staff_scores_length = $scope.staff_scores.length;

		    		//Check how many times the staff has played before, if 3 times stop else proceed
		            if($scope.staff_scores_length < 3){ 

		            	if($scope.staff_scores_length > 0){ //If you have played before

		            		//Check if they have played today. If yes stop else proceed and play the game            	
			            	var last_played = $scope.staff_scores[$scope.staff_scores_length - 1].last_played;

			            	var today = Date.today();
			            	var last_date = Date.parse(last_played);

			            	var diff = Date.compare(today, last_date);		            	

			            	if(diff == 0){ //You can only play once a day			            		

			            		settings.notify('Sorry you can only play again tomorrow');
			            		
			            		$timeout(function(){
			            			$scope.home();
			            		},4000);
			            		
			            	}else{
			            		
			            		$scope.startGame();
			            	} 

		            	}else{ //This is your first time to play
		            		
		            		$scope.startGame();
		            	}
		            	          		

		            }else{
		            	
		            	settings.notify('Sorry you can only play 3 times');
		            	
		            	$timeout(function(){
		        			$scope.home();
		        		},4000);
		            }

		    	}).catch(function(error) {
				    console.log("Error:", error);
				}); //END OF $scope.staff_scores.$loaded()
		      	

	        };//END OF $scope.goPlay


	        $scope.tokenize = function(token,staff){

	        	$scope.authObj.$signInWithCustomToken(token).then(function(authData){			                      	                      
			           
			           var x = staff;
			           x.uid = authData.uid;		           			           

                       properties.setActiveUser(x);
                       $scope.goPlay(authData.uid);

                }).catch(function(error){
					 
					  var errorCode = error.code;
					  var errorMessage = error.message;
					  console.log(errorCode);
					  console.log(errorMessage);
					 
				});

	        };


	        $scope.cancel_login = function() {
	      		$scope.loginstaff = {};	   
	      			
	      		$scope.login_form.$setPristine();
	      		$scope.login_form.$setUntouched();
	      	};	

		  	$scope.player_login = function() {
	            

		        $scope.login_form.submitted = false;
		        	        
		        if ($scope.login_form.$valid) {
		            
		            //Submit form as normal            

		            settings.show_wait();		            

		            var promise = settings.staffLogin($scope.loginstaff);

		            promise.then(function(data) {
			              
			              settings.hide_wait();

			              var response = data.data;

			              if(response.success){
			                    
			                    $scope.logout(); //logout any staff who may be still logged in
			                    $scope.cancel_login();
			                    $scope.tokenize(response.token,response.staff);										                    
			                    

			              }else if(!response.success){
			              	  console.log(response);
			                  settings.notify(response.error);                        
			              }
			          },
			          function(error) {                                  
			              settings.notify(error);
			        });	            
		            
		            
		         } else {
		          	$scope.login_form.submitted = true;          
		         }

	        };//END OF $scope.player_login



	      	$scope.cancel_register = function() {
	      		$scope.newstaff = {};
	      		
	      		$scope.register_form.$setPristine();
	      		$scope.register_form.$setUntouched();
	      	};

	      	$scope.player_register = function() {

		        $scope.register_form.submitted = false;
		        	        
		        if ($scope.register_form.$valid) {
		            
		            //Submit form as normal		            

		            settings.show_wait();
		            
		            //Check if the email address is already taken
		            var status = settings.checkAvailableEmail($scope.newstaff.staff_email, 'staff');

		            status.then(function(data) {
		            	
		            	//console.log(data);

		            	if(data == true){
		            		settings.hide_wait();	
		            		settings.notify('Sorry that email address is already taken');
		            	}else{

		            		//Register the new staff and log them in
		            		//console.log($scope.newstaff.staff_email);
		            		//Get a hash for the password
		            		var promise = settings.getPassword($scope.newstaff.staff_password);
	                
				            promise.then(function(data) {
				                  
				                  var response = data.data;

				                  if(response.success){
				                  	
				                      $scope.newstaff.staff_password = response.password; 
				                      var x = safaricom.createStaff($scope.newstaff);
				                      settings.backupStaff($scope.newstaff); //Local backup

				                      x.then(function(data){

				                      	if(data != false){
					                      	$scope.newstaff.staff_id = data;

					                      	//go login
				                            $scope.newstaff.staff_password = null;

				                            var resp = settings.getStaffToken($scope.newstaff);

				                            resp.then(function(data){
				                                  
				                                  settings.hide_wait();	
				                                  $scope.logout(); //logout any staff who may be still logged in
				                                  var x = data.data;
				                                  
				                                  if(x.success){			                                  		

				                                    $scope.tokenize(x.token,x.staff);

				                                  }else if(!x.success){
				                                      settings.notify(x.error);
				                                  }

				                              },function(error) {
				                              	  console.log(error);
				                                  settings.notify(error);
				                            });

					                    }

				                      },function(error){
						                  console.log(error);
						              });

				                  }else if(!response.success){	                      
				                      settings.notify(response.error);
				                  }
				              },function(error) {                  
				                  settings.notify(error);

				            });

		            	}

		            },function(error) {                                  
                          settings.notify(error);
                    });	
		            
		            
		            
		        } else {
		          $scope.register_form.submitted = true;          
		        }

	        };
	    




	     
	  }//END OF controllers.controller("SignupController",

	]);
});
